@@ -1,23 +1,28 @@
 """PyArr model constants."""
 
+from enum import Enum
+
 CONVERT_TO_BOOL = ("downloadForced",)
 
 CONVERT_TO_FLOAT = (
-    "age",
     "ageHours",
     "ageMinutes",
-    "preferredWordScore",
 )
 
 CONVERT_TO_INTEGER = (
+    "age",
     "fileId",
+    "foreignAuthorId",
     "foreignEditionId",
+    "isbn",
     "isbn13",
-    "protocol",
+    "preferredWordScore",
     "size",
+    "sizeleft",
+    "titleSlug",
+    "trackNumber",
     "tvdbId",
     "tvRageId",
-    "titleSlug",
 )
 
 CONVERT_TO_DATETIME = (
@@ -53,3 +58,16 @@ CONVERT_TO_DATETIME = (
     "stateChangeTime",
     "time",
 )
+
+CONVERT_TO_ENUM = (
+    "preferredProtocol",
+    "protocol",
+)
+
+
+class ProtocolType(Enum):
+    """Protocol type."""
+
+    UNKNOWN = 0
+    USENET = 1
+    TORRENT = 2
