@@ -2,17 +2,45 @@
 from __future__ import annotations
 
 from datetime import datetime
-from typing import TYPE_CHECKING, Any
+from typing import Any
 
-from .const import HTTPMethod
-from .decorator import api_command
-from .request_client import RequestClient
+from aiohttp.client import ClientSession
 
-from .models.radarr import (  # isort:skip
+from aiopyarr.exceptions import ArrException
+from aiopyarr.models.request import Command, SortDirection
+
+from .const import (
+    ALL,
+    DATE,
+    EVENT_TYPE,
+    IS_VALID,
+    MOVIE_ID,
+    NOTIFICATION,
+    PAGE,
+    PAGE_SIZE,
+    PATH,
+    SORT_DIRECTION,
+    SORT_KEY,
+    TERM,
+    TITLE,
+    HTTPMethod,
+)
+from .models.host_configuration import PyArrHostConfiguration
+from .models.radarr import (
+    RadarrAltTitle,
     RadarrBlocklist,
     RadarrBlocklistMovie,
     RadarrCalendar,
+    RadarrCommands,
+    RadarrCredit,
+    RadarrEventType,
+    RadarrExtraFile,
+    RadarrHistory,
     RadarrImportList,
+    RadarrImportListActionType,
+    RadarrImportListMovie,
+    RadarrIndexerFlag,
+    RadarrManualImport,
     RadarrMovie,
     RadarrMovieEditor,
     RadarrMovieFile,
@@ -24,20 +52,18 @@ from .models.radarr import (  # isort:skip
     RadarrQueueDetail,
     RadarrRelease,
     RadarrRename,
+    RadarrRestriction,
+    RadarrSortKeys,
     RadarrTagDetails,
 )
-
-if TYPE_CHECKING:
-    from aiohttp.client import ClientSession
-
-    from .const import HTTPResponse
-    from .models.host_configuration import PyArrHostConfiguration
-    from .models.radarr_common import _RadarrMovieImages
+from .request_client import RequestClient
 
 
 class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     """API client for Radarr endpoints."""
 
+    __name__ = "Radarr"
+
     def __init__(  # pylint: disable=too-many-arguments
         self,
         host_configuration: PyArrHostConfiguration | None = None,
@@ -52,15 +78,14 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         base_api_path: str | None = None,
         request_timeout: float = 60,
         raw_response: bool = False,
-        redact: bool = True,
-        api_ver: str | None = None,
+        api_ver: str = "v3",
     ) -> None:
         """Initialize Radarr API."""
         super().__init__(
             port,
             request_timeout,
             raw_response,
-            redact,
+            api_ver,
             host_configuration,
             session,
             hostname,
@@ -70,58 +95,45 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             ssl,
             verify_ssl,
             base_api_path,
-            api_ver,
         )
 
     async def async_get_movies(
-        self, movieid: int | None = None
+        self,
+        movieid: int | None = None,
+        tmdb: bool = False,
     ) -> RadarrMovie | list[RadarrMovie]:
         """Get information about movies.
 
-        Include a TMDB id for a specific movie or leave black for all.
+        Include an id for a specific movie or leave black for all.
+        tmdb: Use TMDB ID.
         """
         return await self._async_request(
-            f"movie{f'/{movieid}' if movieid is not None else ''}",
+            f"movie{'' if movieid is None or tmdb else f'/{movieid}'}",
+            params=None if movieid is None else {"tmdbid": movieid},
             datatype=RadarrMovie,
         )
 
-    async def async_add_movie(  # pylint: disable=too-many-arguments
-        self,
-        db_id: str,
-        quality_profile_id: int,
-        root_dir: str,
-        monitored: bool = True,
-        search_for_movie: bool = True,
-        tmdb: bool = True,
-    ) -> HTTPResponse:
-        """Add a movie to the database.
-
-        Args:
-            db_id: IMDB or TMDB ID
-            quality_profile_id: ID of the quality profile the movie will use
-            root_dir: location of the root DIR
-            monitored: should the movie be monitored.
-            search_for_movie: Should we search for the movie.
-            tmdb: Use TMDB IDs. Set to False to use IMDB.
-        """
-        movie_json = await self._async_construct_movie_json(
-            db_id, quality_profile_id, root_dir, monitored, search_for_movie, tmdb
-        )
+    async def async_add_movies(
+        self, data: RadarrMovie | list[RadarrMovie]
+    ) -> RadarrMovie | list[RadarrMovie]:
+        """Add movie to the database."""
         return await self._async_request(
-            "movie",
-            params=movie_json,
+            f"movie{'/import' if isinstance(data, list) else ''}",
+            data=data,
+            datatype=RadarrMovie,
             method=HTTPMethod.POST,
         )
 
     async def async_edit_movies(
-        self, data: RadarrMovieEditor, move_files: bool = False
+        self, data: RadarrMovie | RadarrMovieEditor, move_files: bool = False
     ) -> RadarrMovie | list[RadarrMovie]:
         """Edit movie properties of multiple movies at once."""
+        params = {"moveFiles": str(move_files)}
         return await self._async_request(
-            "movie/editor" if hasattr(data, "movieIds") else "movie",
-            params=None if hasattr(data, "movieIds") else {"moveFiles": move_files},
+            f"movie{'' if isinstance(data, RadarrMovie) else '/editor'}",
+            params=params if isinstance(data, RadarrMovie) else None,
             data=data,
-            datatype=RadarrMovieEditor,
+            datatype=RadarrMovie,
             method=HTTPMethod.PUT,
         )
 
@@ -130,19 +142,21 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         ids: int | list[int],
         delete_files: bool = False,
         add_exclusion: bool = False,
-    ) -> HTTPResponse:
+    ) -> None:
         """Delete movies (and optionally files).
 
         ids: include an integer to delete one movie or a list for mass deletion
         """
-        data = {
-            "movieIds": ids,
-            "deleteFiles": delete_files,
-            "addImportExclusion": add_exclusion,
+        data: dict[str, str | list[int]] = {
+            "deleteFiles": str(delete_files),
+            "addImportExclusion": str(add_exclusion),
         }
+        if isinstance(ids, list):
+            data["movieIds"] = ids
         return await self._async_request(
             "movie/editor" if isinstance(ids, list) else f"movie/{ids}",
-            data=data,
+            params=None if isinstance(ids, list) else data,
+            data=data if isinstance(ids, list) else None,
             method=HTTPMethod.DELETE,
         )
 
@@ -158,15 +172,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             method=HTTPMethod.POST,
         )
 
-    async def async_get_movie_files_by_movie_id(self, movieid: int) -> RadarrMovieFile:
-        """Get a movie file object by Movie database id."""
-        return await self._async_request(
-            "moviefile",
-            params={"movieid": movieid},
-            datatype=RadarrMovieFile,
-        )
-
-    async def async_delete_movie_file(self, movieid: int) -> HTTPResponse:
+    async def async_delete_movie_file(self, movieid: int) -> None:
         """Delete a moviefile by its database id."""
         return await self._async_request(
             f"moviefile/{movieid}",
@@ -182,57 +188,71 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """
         return await self._async_request(
             "movie/lookup",
-            params={"term": f"{'tmdb' if tmdb is True else 'imdb'} :{term}"},
+            params={TERM: f"{'tmdb' if tmdb else 'imdb'}:{term}"},
             datatype=RadarrMovie,
         )
 
-    async def async_lookup_movie_files(self, ids: list[int]) -> list[RadarrMovieFile]:
+    async def async_lookup_movie_files(
+        self, ids: list[int]
+    ) -> RadarrMovieFile | list[RadarrMovieFile]:
         """Get movie file information for multiple movie files."""
         return await self._async_request(
-            "moviefile", params={"moviefileids": ids}, datatype=RadarrMovieFile
+            f"moviefile{'' if isinstance(ids, list) else f'/{ids}'}",
+            params={"movieFileIds": ids} if isinstance(ids, list) else None,
+            datatype=RadarrMovieFile,
         )
 
     async def async_get_history(
         self,
         page: int = 1,
         page_size: int = 20,
-        sort_direction: str = "descending",
-        sort_key: str = "date",
-    ) -> list[RadarrMovieHistory]:
-        """Return a json object list of items in your history.
+        sort_key: RadarrSortKeys = RadarrSortKeys.DATE,
+        event_type: RadarrEventType | None = None,
+    ) -> RadarrHistory:
+        """Get movie history.
 
         Args:
             page: Page to be returned.
             page_size: Number of results per page.
-            sort_direction: Direction to sort items.
-            sort_key: Field to sort by.
+            sort_key: date, id, movieid, title, sourcetitle, path, ratings, or quality
+                    (Others do not apply)
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_KEY: sort_key.value,
         }
+        if event_type and event_type in RadarrEventType:
+            params[EVENT_TYPE] = event_type.value
         return await self._async_request(
             "history",
             params=params,
-            datatype=RadarrMovieHistory,
+            datatype=RadarrHistory,
         )
 
-    async def async_get_movie_history(
-        self, recordid: int, event_type: int | None = None
+    async def async_get_history_since(
+        self,
+        date: datetime | None = None,
+        movieid: int | None = None,
+        event_type: RadarrEventType | None = None,
     ) -> list[RadarrMovieHistory]:
-        """Get history for a given movie in database by its database id.
+        """Get history since specified date.
 
-        Args:
-            id: Database id of movie
-            event_type: History event type to retrieve.
+        movieid: include to search history by movie id (date will not apply)
+        Radarr permits a naked query but its required here to avoid excessively large
+        data sets where filtering should be used instead
         """
-        params = {"movieId": recordid}
-        if event_type:
-            params["eventType"] = event_type
+        if date is None and movieid is None:
+            raise ArrException(self, "Either date or movieid is required")
+        params: dict[str, int | str] = {}
+        if isinstance(date, datetime):
+            params[DATE] = date.strftime("%Y-%m-%d")
+        elif movieid is not None:
+            params[MOVIE_ID] = movieid
+        if event_type and event_type in RadarrEventType:
+            params[EVENT_TYPE] = event_type.value
         return await self._async_request(
-            "history/movie",
+            f"history/{'since' if isinstance(date, datetime) else 'movie'}",
             params=params,
             datatype=RadarrMovieHistory,
         )
@@ -240,21 +260,16 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     async def async_get_import_lists(
         self, listid: int | None = None
     ) -> RadarrImportList | list[RadarrImportList]:
-        """Get information about import list.
-
-        id: Get import list matching id. Leave blank for all.
-        """
+        """Get information about import lists."""
         return await self._async_request(
-            f"importlist{f'/{listid}' if listid is not None else ''}",
+            f"importlist{'' if listid is None else f'/{listid}'}",
             datatype=RadarrImportList,
         )
 
-    async def async_edit_import_list(
-        self, listid: int, data: RadarrImportList
-    ) -> RadarrImportList:
+    async def async_edit_import_list(self, data: RadarrImportList) -> RadarrImportList:
         """Edit an importlist."""
         return await self._async_request(
-            f"importlist/{listid}",
+            "importlist",
             data=data,
             datatype=RadarrImportList,
             method=HTTPMethod.PUT,
@@ -266,9 +281,114 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             "importlist", data=data, datatype=RadarrImportList, method=HTTPMethod.POST
         )
 
-    @api_command("config/naming", datatype=RadarrNamingConfig)
+    async def async_test_import_lists(
+        self, data: RadarrImportList | None = None
+    ) -> bool:
+        """Test all import lists."""
+        _res = await self._async_request(
+            f"importlist/test{ALL if data is None else ''}",
+            data=None if data is None else data,
+            method=HTTPMethod.POST,
+        )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
+
+    async def async_get_import_list_movies(self) -> list[RadarrImportListMovie]:
+        """Get list of movies on configured import lists."""
+        return await self._async_request(
+            "importlist/movie",
+            datatype=RadarrImportListMovie,
+        )
+
+    async def async_get_extra_file(self, movieid: int) -> list[RadarrExtraFile]:
+        """Get extra files info from specified movie id."""
+        return await self._async_request(
+            "extrafile",
+            params={MOVIE_ID: movieid},
+            datatype=RadarrExtraFile,
+        )
+
+    async def async_get_restrictions(
+        self, restrictionid: int | None = None
+    ) -> RadarrRestriction | list[RadarrRestriction]:
+        """Get indexer restrictions."""
+        return await self._async_request(
+            f"restriction{'' if restrictionid is None else f'/{restrictionid}'}",
+            datatype=RadarrRestriction,
+        )
+
+    async def async_edit_restriction(
+        self, data: RadarrRestriction
+    ) -> RadarrRestriction:
+        """Edit indexer restriction."""
+        return await self._async_request(
+            "restriction",
+            data=data,
+            datatype=RadarrRestriction,
+            method=HTTPMethod.PUT,
+        )
+
+    async def async_add_restriction(self, data: RadarrRestriction) -> RadarrRestriction:
+        """Add indexer restriction."""
+        return await self._async_request(
+            "restriction",
+            data=data,
+            datatype=RadarrRestriction,
+            method=HTTPMethod.POST,
+        )
+
+    async def async_delete_restriction(self, restrictionid: int) -> None:
+        """Delete indexer restriction."""
+        return await self._async_request(
+            f"restriction/{restrictionid}",
+            datatype=RadarrRestriction,
+            method=HTTPMethod.DELETE,
+        )
+
+    async def async_get_credits(
+        self, creditid: int | None = None, movieid: int | None = None
+    ) -> RadarrCredit | list[RadarrCredit]:
+        """Get credits."""
+        return await self._async_request(
+            f"credit{'' if creditid is None else f'/{creditid}'}",
+            params=None if movieid is None else {MOVIE_ID: movieid},
+            datatype=RadarrCredit,
+        )
+
+    async def async_get_alt_titles(
+        self, alttitleid: int | None = None, movieid: int | None = None
+    ) -> RadarrAltTitle | list[RadarrAltTitle]:
+        """Get alternate movie titles."""
+        return await self._async_request(
+            f"alttitle{'' if alttitleid is None else f'/{alttitleid}'}",
+            params=None if movieid is None else {MOVIE_ID: movieid},
+            datatype=RadarrAltTitle,
+        )
+
+    # GET altyear
+
+    async def async_get_indexer_flags(self) -> list[RadarrIndexerFlag]:
+        """Get indexer flags."""
+        return await self._async_request(
+            "indexerflag",
+            datatype=RadarrIndexerFlag,
+        )
+
+    async def async_importlist_action(
+        self, action: RadarrImportListActionType
+    ) -> dict[str, Any]:
+        """Perform import list action."""
+        return await self._async_request(
+            f"importlist/action/{action.value}",
+            method=HTTPMethod.POST,
+        )
+
     async def async_get_naming_config(self) -> RadarrNamingConfig:
         """Get information about naming configuration."""
+        return await self._async_request("config/naming", datatype=RadarrNamingConfig)
 
     async def async_edit_naming_config(
         self, data: RadarrNamingConfig
@@ -289,7 +409,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get tag details matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"tag/detail{f'/{tagid}' if tagid is not None else ''}",
+            f"tag/detail{'' if tagid is None else f'/{tagid}'}",
             datatype=RadarrTagDetails,
         )
 
@@ -297,22 +417,22 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         self,
         page: int = 1,
         page_size: int = 20,
-        sort_direction: str = "descending",
-        sort_key: str = "date",
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: RadarrSortKeys = RadarrSortKeys.DATE,
     ) -> RadarrBlocklist:
         """Return blocklisted releases.
 
         Args:
             page: Page to be returned.
             page_size: Number of results per page.
-            sort_direction: Direction to sort items.
-            sort_key: Field to sort by.
+            sort_key: date, id, movieid, title, path, sourcetitle, ratings, or quality
+                    (Others do not apply)
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
         }
         return await self._async_request(
             "blocklist",
@@ -327,32 +447,16 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Retrieve blocklisted releases that are tied to a given movie in the database."""
         return await self._async_request(
             "blocklist/movie",
-            params={"movieId": bocklistid},
+            params={MOVIE_ID: bocklistid},
             datatype=RadarrBlocklistMovie,
         )
 
-    async def async_delete_blocklist(self, bocklistid: int) -> HTTPResponse:
-        """Remove a specific release (the id provided) from the blocklist."""
-        return await self._async_request(
-            "blocklist",
-            params={"id": bocklistid},
-            method=HTTPMethod.DELETE,
-        )
-
-    async def async_delete_blocklist_bulk(self, data: list[int]) -> HTTPResponse:
-        """Delete blocklisted releases in bulk."""
-        return await self._async_request(
-            "blocklist/bulk",
-            data=data,
-            method=HTTPMethod.DELETE,
-        )
-
     async def async_get_queue(  # pylint: disable=too-many-arguments
         self,
         page: int = 1,
         page_size: int = 20,
-        sort_direction: str = "ascending",
-        sort_key: str = "timeLeft",
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: RadarrSortKeys = RadarrSortKeys.TIMELEFT,
         include_unknown_movie_items: bool = False,
         include_movie: bool = False,
     ) -> RadarrQueue:
@@ -361,16 +465,14 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         Args:
             page: Page to be returned.
             page_size: Number of results per page.
-            sort_direction: Direction to sort items.
-            sort_key: Field to sort by.
             include_unknown_movie_items: Include unknown movie items.
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
-            "includeUnknownMovieItems": str(include_unknown_movie_items),  # Unverified
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
+            "includeUnknownMovieItems": str(include_unknown_movie_items),
             "includeMovie": str(include_movie),
         }
         return await self._async_request("queue", params=params, datatype=RadarrQueue)
@@ -382,7 +484,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> list[RadarrQueueDetail]:
         """Get details of all items in queue."""
         params = {
-            "includeUnknownMovieItems": str(include_unknown_movie_items),  # Unverified
+            "includeUnknownMovieItems": str(include_unknown_movie_items),
             "includeMovie": str(include_movie),
         }
         return await self._async_request(
@@ -399,7 +501,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get notification matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"notification{f'/{notifyid}' if notifyid is not None else ''}",
+            f"notification{'' if notifyid is None else f'/{notifyid}'}",
             datatype=RadarrNotification,
         )
 
@@ -408,7 +510,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> RadarrNotification:
         """Edit a notification."""
         return await self._async_request(
-            f"notification/{data.id}",
+            NOTIFICATION,
             data=data,
             datatype=RadarrNotification,
             method=HTTPMethod.PUT,
@@ -419,58 +521,85 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> RadarrNotification:
         """Add a notification."""
         return await self._async_request(
-            "notification",
+            NOTIFICATION,
             data=data,
             datatype=RadarrNotification,
             method=HTTPMethod.POST,
         )
 
-    async def async_test_notification(self, data: RadarrNotification) -> HTTPResponse:
+    async def async_test_notifications(
+        self, data: RadarrNotification | None = None
+    ) -> bool:
         """Test a notification configuration."""
-        return await self._async_request(
-            "notification/test",
-            data=data,
+        _res = await self._async_request(
+            f"notification/test{ALL if data is None else ''}",
+            data=None if data is None else data,
             method=HTTPMethod.POST,
         )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
 
     async def async_parse(self, title: str) -> RadarrParse:
         """Return the movie with matching file name."""
-        params = {"title": title}
+        params = {TITLE: title}
         return await self._async_request("parse", params=params, datatype=RadarrParse)
 
-    async def async_command_downloaded_movies_scan(self) -> HTTPResponse:
-        """Trigger the scan of downloaded movies."""
-        return await self._async_request(
-            "command",
-            data={"name": "DownloadedMoviesScan"},
-            method=HTTPMethod.POST,
-        )
+    async def async_radarr_command(  # pylint: disable=too-many-arguments
+        self,
+        command: RadarrCommands,
+        clientid: int | None = None,
+        copymode: bool = True,
+        files: list[int] | None = None,
+        path: str | None = None,
+        movieid: int | list[int] | None = None,
+    ) -> Command:
+        """Send a command to Radarr.
 
-    async def async_command_missing_movies_search(self) -> HTTPResponse:
-        """Trigger a search of all missing movies."""
+        Specify clientid for DownloadedMoviesScan (Optional)
+        Specify files for RenameFiles
+        Specify path for DownloadedMoviesScan (Optional)
+        Specify movieid for:
+            RefreshMovie (Optional),
+            RenameMovie (list[int]),
+            RescanMovie (Optional),
+            MovieSearch
+        """
+        data: dict[str, str | int | list[int]] = {"name": command.value}
+        if clientid is not None:
+            data["downloadClientId"] = clientid
+        if files is not None:
+            data["files"] = files
+        if path is not None:
+            data[PATH] = path
+        if movieid is not None:
+            if command == RadarrCommands.RENAME_MOVIE:
+                data["movieIds"] = movieid
+            else:
+                data[MOVIE_ID] = movieid
+        if command is RadarrCommands.DOWNLOADED_MOVIES_SCAN:
+            data["importMode"] = "Copy" if copymode else "Move"
         return await self._async_request(
             "command",
-            data={"name": "MissingMoviesSearch"},
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_refresh_movies(self, movieids: list[int]) -> HTTPResponse:
-        """Trigger a refresh / scan of movies."""
-        return await self._async_request(
-            "command",
-            data={"name": "RefreshMovie", "movieIds": movieids},
+            data=data,
+            datatype=Command,
             method=HTTPMethod.POST,
         )
 
     async def async_get_calendar(
-        self, start_date: datetime, end_date: datetime, unmonitored: bool = True
+        self,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        unmonitored: bool = True,
     ) -> list[RadarrCalendar]:
         """Get a list of movies based on calendar parameters."""
-        params = {
-            "start": start_date.strftime("%Y-%m-%d"),
-            "end": end_date.strftime("%Y-%m-%d"),
-            "unmonitored": str(unmonitored),
-        }
+        params = {"unmonitored": str(unmonitored)}
+        if start_date:
+            params["start"] = start_date.strftime("%Y-%m-%d")
+        if end_date:
+            params["end"] = end_date.strftime("%Y-%m-%d")
         return await self._async_request(
             "calendar",
             params=params,
@@ -483,7 +612,7 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Search indexers for specified fields."""
         return await self._async_request(
             "release",
-            params={"movieId": movieid} if movieid is not None else None,
+            params=None if movieid is None else {MOVIE_ID: movieid},
             datatype=RadarrRelease,
         )
 
@@ -518,57 +647,36 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Get files matching specified id that are not properly renamed yet."""
         return await self._async_request(
             "rename",
-            params={"movieId": movieid},
+            params={MOVIE_ID: movieid},
             datatype=RadarrRename,
         )
 
-    async def _async_construct_movie_json(  # pylint: disable=too-many-arguments
+    async def async_get_manual_import(
         self,
-        db_id: str,
-        quality_profile_id: int,
-        root_dir: str,
-        monitored: bool = True,
-        search_for_movie: bool = True,
-        tmdb: bool = True,
-    ) -> dict[str, str | int | dict[str, bool] | list[_RadarrMovieImages]] | None:
-        """Search for movie on tmdb and returns Movie json to add.
-
-        Args:
-            db_id: imdb or tmdb id
-            quality_profile_id: ID of the quality profile the movie will use
-            root_dir: location of the root DIR
-            monitored: should the movie be monitored.
-            search_for_movie: Should we search for the movie.
-            tmdb: Use TMDB IDs. Set to False to use IMDB.
-        """
-        if not (movie := await self.async_lookup_movie(db_id, tmdb=tmdb)):
-            raise Exception("Movie Doesn't Exist")
-
-        return {
-            "title": movie[0].title or "",
-            "rootFolderPath": root_dir,
-            "qualityProfileId": quality_profile_id,
-            "year": movie[0].year or "",
-            "tmdbId": movie[0].tmdbId or "",
-            "images": movie[0].images or [],
-            "titleSlug": movie[0].titleSlug or "",
-            "monitored": monitored,
-            "addOptions": {"searchForMovie": search_for_movie},
+        downloadid: str,
+        folder: str | None = None,
+        filterexistingfiles: bool = True,
+    ) -> list[RadarrManualImport]:
+        """Get manual import."""
+        params = {
+            "downloadId": downloadid,
+            "filterExistingFiles": str(filterexistingfiles),
+            "folder": folder if folder is not None else "",
         }
+        return await self._async_request(
+            "manualimport", params=params, datatype=RadarrManualImport
+        )
 
-    async def async_get_import_list_exclusions(
-        self, clientid: int | None = None
-    ) -> Any:
-        """Get import list exclusions."""
-        raise NotImplementedError()
-
-    async def async_delete_import_list_exclusion(self, clientid: int) -> Any:
-        """Delete import list exclusion."""
-        raise NotImplementedError()
-
-    async def async_add_import_list_exclusion(self, data: Any) -> Any:
-        """Add import list exclusion."""
-        raise NotImplementedError()
+    async def async_edit_manual_import(
+        self, data: RadarrManualImport
+    ) -> list[RadarrManualImport]:
+        """Get manual import."""
+        return await self._async_request(
+            "manualimport",
+            data=data,
+            datatype=RadarrManualImport,
+            method=HTTPMethod.PUT,
+        )
 
     async def async_get_release_profiles(self, profileid: int | None = None) -> Any:
         """Get release profiles."""
@@ -582,10 +690,10 @@ class RadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Delete release profiles."""
         raise NotImplementedError()
 
-    async def async_add_release_profiles(self, data: Any) -> Any:
+    async def async_add_release_profile(self, data: Any) -> Any:
         """Add release profile."""
         raise NotImplementedError()
 
-    async def async_edit_import_list_exclusion(self, data: Any) -> Any:
-        """Edit import list exclusion."""
+    async def async_delete_metadata_profile(self, profileid: int) -> Any:
+        """Delete a metadata profile."""
         raise NotImplementedError()
