@@ -1,15 +1,31 @@
 """Readarr API."""
 from __future__ import annotations
 
-import json
 from datetime import datetime
-from typing import TYPE_CHECKING
 
-from .const import HTTPMethod, HTTPResponse
-from .decorator import api_command
-from .request_client import RequestClient
+from aiohttp.client import ClientSession
 
-from .models.readarr import (  # isort:skip
+from aiopyarr.exceptions import ArrException
+from aiopyarr.models.request import Command, Indexer, RootFolder, SortDirection
+
+from .const import (
+    ALL,
+    AUTHOR_ID,
+    BOOK_ID,
+    DATE,
+    EVENT_TYPE,
+    IS_VALID,
+    NOTIFICATION,
+    PAGE,
+    PAGE_SIZE,
+    SORT_DIRECTION,
+    SORT_KEY,
+    TERM,
+    TITLE,
+    HTTPMethod,
+)
+from .models.host_configuration import PyArrHostConfiguration
+from .models.readarr import (
     ReadarrAuthor,
     ReadarrAuthorEditor,
     ReadarrAuthorLookup,
@@ -17,15 +33,20 @@ from .models.readarr import (  # isort:skip
     ReadarrBook,
     ReadarrBookFile,
     ReadarrBookFileEditor,
+    ReadarrBookHistory,
     ReadarrBookLookup,
     ReadarrBookshelf,
+    ReadarrBookTypes,
     ReadarrCalendar,
-    ReadarrDelayProfile,
+    ReadarrCommands,
     ReadarrDevelopmentConfig,
+    ReadarrEventType,
     ReadarrHistory,
-    ReadarrImportlist,
+    ReadarrImportList,
+    ReadarrImportListOptions,
+    ReadarrManualImport,
     ReadarrMetadataProfile,
-    ReadarrMetadataProviderConfig,
+    ReadarrMetadataProvider,
     ReadarrNamingConfig,
     ReadarrNotification,
     ReadarrParse,
@@ -36,20 +57,19 @@ from .models.readarr import (  # isort:skip
     ReadarrRetag,
     ReadarrSearch,
     ReadarrSeries,
+    ReadarrSortKeys,
     ReadarrTagDetails,
     ReadarrWantedCutoff,
     ReadarrWantedMissing,
 )
-
-if TYPE_CHECKING:
-    from aiohttp.client import ClientSession
-
-    from .models.host_configuration import PyArrHostConfiguration
+from .request_client import RequestClient
 
 
 class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     """API client for Readarr endpoints."""
 
+    __name__ = "Readarr"
+
     def __init__(  # pylint: disable=too-many-arguments
         self,
         host_configuration: PyArrHostConfiguration | None = None,
@@ -64,7 +84,6 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         base_api_path: str | None = None,
         request_timeout: float = 10,
         raw_response: bool = False,
-        redact: bool = True,
         api_ver: str = "v1",
     ) -> None:
         """Initialize Readarr API."""
@@ -72,7 +91,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             port,
             request_timeout,
             raw_response,
-            redact,
+            api_ver,
             host_configuration,
             session,
             hostname,
@@ -82,144 +101,67 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             ssl,
             verify_ssl,
             base_api_path,
-            api_ver,
         )
 
-    async def async_author(
+    async def async_get_authors(
         self, authorid: int | None = None
     ) -> ReadarrAuthor | list[ReadarrAuthor]:
-        """Get info about specified author by id or using a term for lookup."""
-        command = f"author{f'/{authorid}' if authorid is not None else ''}"
+        """Get info about specified author by id, leave blank for all."""
+        command = f"author{'' if authorid is None else f'/{authorid}'}"
         return await self._async_request(command, datatype=ReadarrAuthor)
 
     async def async_author_lookup(self, term: str) -> list[ReadarrAuthorLookup]:
         """Search for new authors using a term."""
         return await self._async_request(
-            "author/lookup", params={"term": term}, datatype=ReadarrAuthorLookup
+            "author/lookup", params={TERM: term}, datatype=ReadarrAuthorLookup
         )
 
-    async def async_author_add(self, data: ReadarrAuthor) -> ReadarrAuthor:
+    async def async_add_author(self, data: ReadarrAuthor) -> ReadarrAuthor:
         """Add author to database."""
         return await self._async_request(
             "author", data=data, datatype=ReadarrAuthor, method=HTTPMethod.POST
         )
 
-    async def async_author_edit(self, data: ReadarrAuthorEditor) -> ReadarrAuthorEditor:
-        """Add author database info."""
+    async def async_edit_authors(
+        self, data: ReadarrAuthor | ReadarrAuthorEditor
+    ) -> ReadarrAuthor | list[ReadarrAuthor]:
+        """Edit author database info."""
         return await self._async_request(
-            "author/editor",
+            f"author{'' if isinstance(data, ReadarrAuthor) else '/editor'}",
             data=data,
-            datatype=ReadarrAuthorEditor,
+            datatype=ReadarrAuthor,
             method=HTTPMethod.PUT,
         )
 
-    # add author editor delete
-    async def async_delete_author(
+    async def async_delete_authors(
         self,
-        authorid: int,
+        authorids: int | list[int],
         delete_files: bool = False,
         import_list_exclusion: bool = True,
-    ) -> HTTPResponse:
+    ) -> None:
         """Delete the author with the given id.
 
         Args:
-            authorid: Database id for author
+            authorids: Database ids for author (authorMetadataId)
             delete_files: If true author folder and files will be deleted
             import_list_exclusion: Add an exclusion so author doesn't get re-added
         """
-        params = {
-            "deleteFiles": delete_files,
-            "addImportListExclusion": import_list_exclusion,
+        data = {
+            "deleteFiles": str(delete_files),
+            "addImportListExclusion": str(import_list_exclusion),
         }
         return await self._async_request(
-            f"author/{authorid}", params=params, method=HTTPMethod.DELETE
+            f"author/{'editor' if isinstance(authorids, list) else authorids}",
+            data=data,
+            method=HTTPMethod.DELETE,
         )
 
-    async def _async_construct_book_json(  # pylint: disable=too-many-arguments
-        self,
-        db_id: int,
-        book_id_type: str,
-        root_dir: str,
-        quality_profile_id: int = 1,
-        metadata_profile_id: int = 0,
-        monitored: bool = True,
-        search_for_new_book: bool = False,
-        author_monitor: str = "all",
-        author_search_for_missing_books: bool = False,
-    ) -> ReadarrBookLookup | dict | None:
-        """Construct the JSON required to add a new book to Readarr.
-
-        Args:
-            db_id: goodreads, isbn, asin ID
-            book_id_type: goodreads / isbn / asin
-            root_dir: root directory for books
-            quality_profile_id: quality profile id
-            metadata_profile_id: metadata profile id
-            monitored: should the book be monitored
-            search_for_new_book : shour a search for the new book happen
-            author_monitor: monitor the author.
-            author_search_for_missing_books: search for other missing books by the author
-        """
-        book_id_types = ["goodreads", "isbn", "asin"]
-        if book_id_type not in book_id_types:
-            raise ValueError(f"Invalid book id type. Expected one of: {book_id_types}")
-
-        books = await self.async_lookup_book(book_id_type + ":" + str(db_id))
-        if not isinstance(books, list) or books[0].author is None:
-            return None
-
-        books[0].author.metadataProfileId = metadata_profile_id
-        books[0].author.qualityProfileId = quality_profile_id
-        books[0].author.rootFolderPath = root_dir
-        if books[0].author.addOptions:
-            books[0].author.addOptions.monitor = author_monitor
-            books[
-                0
-            ].author.addOptions.searchForMissingBooks = author_search_for_missing_books
-        books[0].monitored = monitored
-
-        books[0].author.__setattr__("manualAdd", True)
-        books[0].__setattr__("addOptions", {"searchForNewBook": search_for_new_book})
-
-        return books[0]
-
-    async def async_command_check_updates(self) -> HTTPResponse:
-        """Trigger checking application updates."""
+    async def async_readarr_command(self, command: ReadarrCommands) -> Command:
+        """Send a command to Readarr."""
         return await self._async_request(
             "command",
-            data={"name": "ApplicationUpdateCheck"},
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_backup(self) -> HTTPResponse:
-        """Trigger application backup."""
-        return await self._async_request(
-            "command",
-            data={"name": "Backup"},
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_check_health(self) -> HTTPResponse:
-        """Trigger a system health check."""
-        return await self._async_request(
-            "command",
-            data={"name": "CheckHealth"},
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_refresh_author(self) -> HTTPResponse:
-        """Trigger author refresh."""
-        return await self._async_request(
-            "command",
-            data={"name": "RefreshAuthor"},
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_rescan_folders(self) -> HTTPResponse:
-        """Trigger folder rescan."""
-        return await self._async_request(
-            "command",
-            data={"name": "RescanFolders"},
+            data={"name": command.value},
+            datatype=Command,
             method=HTTPMethod.POST,
         )
 
@@ -227,22 +169,22 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         self,
         page: int = 1,
         page_size: int = 20,
-        sort_direction: str = "descending",
-        sort_key: str = "date",
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: ReadarrSortKeys = ReadarrSortKeys.DATE,
     ) -> ReadarrBlocklist:
         """Return blocklisted releases.
 
         Args:
             page: Page to be returned.
             page_size: Number of results per page.
-            sort_direction: Direction to sort items.
-            sort_key: Field to sort by.
+            sort_key: date, id, authorid, sourcetitle, quality,
+            indexer, path, message, or ratings. (Others do not apply)
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
         }
         return await self._async_request(
             "blocklist",
@@ -253,60 +195,58 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     async def async_get_wanted_missing(  # includeAuthor, sortDir not working
         self,
         recordid: int | None = None,
-        sortkey: str = "title",
+        sort_key: ReadarrSortKeys = ReadarrSortKeys.TITLE,
         page: int = 1,
         page_size: int = 10,
     ) -> ReadarrWantedMissing | ReadarrBook:
         """Get missing books.
 
         Args:
-            sortkey: id, title, ratings, or others".
+            sort_key: id, title, ratings, bookid, or quality. (Others do not apply)
             page: Page number to return.
             page_size: Number of items per page.
         """
         params = {
-            "sortKey": sortkey,
-            "page": page,
-            "pageSize": page_size,
+            SORT_KEY: sort_key.value,
+            PAGE: page,
+            PAGE_SIZE: page_size,
         }
         return await self._async_request(
-            f"wanted/missing{f'/{recordid}' if recordid is not None else ''}",
+            f"wanted/missing{'' if recordid is None else f'/{recordid}'}",
             params=params,
-            datatype=ReadarrBook if recordid is not None else ReadarrWantedMissing,
+            datatype=ReadarrWantedMissing if recordid is None else ReadarrBook,
         )
 
     async def async_get_wanted_cutoff(  # includeAuthor, sortDir not working
         self,
         recordid: int | None = None,
-        sortkey: str = "title",
+        sort_key: ReadarrSortKeys = ReadarrSortKeys.TITLE,
         page: int = 1,
         page_size: int = 10,
     ) -> ReadarrWantedCutoff | ReadarrBook:
         """Get wanted books not meeting cutoff.
 
         Args:
-            sortkey: id, title, ratings, or others".
+            sort_key: id, title, ratings, bookid, or quality". (others do not apply)
             page: Page number to return.
             page_size: Number of items per page.
-            sort_asc: Sort items in ascending order.
         """
         params = {
-            "sortKey": sortkey,
-            "page": page,
-            "pageSize": page_size,
+            SORT_KEY: sort_key.value,
+            PAGE: page,
+            PAGE_SIZE: page_size,
         }
         return await self._async_request(
-            f"wanted/cutoff{f'/{recordid}' if recordid is not None else ''}",
+            f"wanted/cutoff{'' if recordid is None else f'/{recordid}'}",
             params=params,
-            datatype=ReadarrBook if recordid is not None else ReadarrWantedCutoff,
+            datatype=ReadarrWantedCutoff if recordid is None else ReadarrBook,
         )
 
     async def async_get_queue(  # pylint: disable=too-many-arguments
         self,
         page: int = 1,
         page_size: int = 10,
-        sort_dir: str = "ascending",
-        sort_key: str = "timeleft",
+        sort_key: ReadarrSortKeys = ReadarrSortKeys.TIMELEFT,
         unknown_authors: bool = False,
         include_author: bool = False,
         include_book: bool = False,
@@ -316,18 +256,14 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         Args:
             page: page number
             page_size: number of results per page_size
-            sort_dir: direction to sort
-            sort_key: field to sort by
             unknown_authors: Include items with an unknown author
             include_author: Include the author
             include_book: Include the book
         """
-
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_dir,
-            "sortKey": sort_key,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_KEY: sort_key.value,
             "includeUnknownAuthorItems": str(unknown_authors),
             "includeAuthor": str(include_author),
             "includeBook": str(include_book),
@@ -349,7 +285,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             "includeBook": str(include_book),
         }
         if authorid is not None:
-            params["authorId"] = authorid
+            params[AUTHOR_ID] = authorid
         if bookids is not None:
             params["bookIds"] = bookids
         return await self._async_request(
@@ -362,19 +298,19 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         self, bookid: int | None = None
     ) -> ReadarrBook | list[ReadarrBook]:
         """Return all books in your collection or book with matching book ID."""
-        path = f"book/{bookid}" if bookid is not None else "book"
+        path = f"book{'' if bookid is None else f'/{bookid}'}"
         return await self._async_request(path, datatype=ReadarrBook)
 
     async def async_add_book(self, data: ReadarrBook) -> ReadarrBook:
         """Add a new book and its associated author (if not already added)."""
         return await self._async_request(
-            "book", data=json.dumps(data), method=HTTPMethod.POST
+            "book", data=data, datatype=ReadarrBook, method=HTTPMethod.POST
         )
 
     async def async_edit_book(self, data: ReadarrBook) -> ReadarrBook:
         """Edit an existing book (if not already added)."""
         return await self._async_request(
-            f"book/{data.id}", data=data, datatype=ReadarrBook, method=HTTPMethod.PUT
+            "book", data=data, datatype=ReadarrBook, method=HTTPMethod.PUT
         )
 
     async def async_delete_book(
@@ -382,7 +318,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         bookid: int,
         delete_files: bool = False,
         import_list_exclusion: bool = True,
-    ) -> HTTPResponse:
+    ) -> None:
         """Delete the book with the given id.
 
         Args:
@@ -391,8 +327,8 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             import_list_exclusion: Add an exclusion so book doesn't get re-added
         """
         params = {
-            "deleteFiles": delete_files,
-            "addImportListExclusion": import_list_exclusion,
+            "deleteFiles": str(delete_files),
+            "addImportListExclusion": str(import_list_exclusion),
         }
         return await self._async_request(
             f"book/{bookid}", params=params, method=HTTPMethod.DELETE
@@ -401,189 +337,214 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     async def async_get_book_file(
         self,
         authorid: int | None = None,
-        fileid: int | list[int] | None = None,
         bookid: list[int] | None = None,
+        fileid: int | list[int] | None = None,
         unmapped: bool = False,
     ) -> ReadarrBookFile:
-        """Return all books in your collection or book with matching book ID."""
-        params = {"unmapped": str(unmapped)}
+        """Return all books in your collection or book with matching book ID.
+
+        unmapped: True to instead get all files not matched to known books
+        """
+        params: dict[str, str | int | list[int]] = {"unmapped": str(unmapped)}
         if not isinstance(fileid, int):
             if authorid is not None:
-                params["authorId"] = str(authorid)
-            if fileid is not None:
-                params["bookFileIds"] = str(fileid)
+                params[AUTHOR_ID] = authorid
             if bookid is not None:
-                params["bookId"] = str(bookid)
+                params[BOOK_ID] = bookid
+            if fileid is not None:
+                params["bookFileIds"] = fileid
         if isinstance(fileid, list):
             path = "bookfile"
         else:
-            path = f"bookfile/{fileid}" if fileid is not None else "bookfile"
+            path = f"bookfile{'' if fileid is None else f'/{fileid}'}"
         return await self._async_request(path, params=params, datatype=ReadarrBookFile)
 
-    async def async_edit_book_file(self, data: ReadarrBookFile) -> ReadarrBookFile:
+    async def async_edit_book_files(
+        self, data: ReadarrBookFile | ReadarrBookFileEditor
+    ) -> ReadarrBookFile | list[ReadarrBookFile]:
         """Edit book file attributes."""
         return await self._async_request(
-            f"bookfile/{data.id}",
+            f"bookfile{'' if isinstance(data, ReadarrBookFile) else '/editor'}",
             data=data,
             datatype=ReadarrBookFile,
             method=HTTPMethod.PUT,
         )
 
-    async def async_edit_book_file_bulk(
-        self, data: ReadarrBookFileEditor
-    ) -> HTTPResponse:
-        """Epdate book file attributes in bulk."""
+    async def async_delete_book_files(self, ids: int | list[int]) -> None:
+        """Delete book files. Use integer for one file or list for mass deletion."""
         return await self._async_request(
-            "bookfile/editor",
-            data=data,
-            method=HTTPMethod.PUT,
+            f"bookfile/{'bulk' if isinstance(ids, list) else f'{ids}'}",
+            data={"bookFileIds": ids} if isinstance(ids, list) else None,
+            method=HTTPMethod.DELETE,
         )
 
-    async def async_delete_book_file(
-        self, fileid: int | None = None, data: ReadarrBookFileEditor | None = None
-    ) -> HTTPResponse:
-        """Delete book files. Use fileid for one file or data for mass deletion."""
-        command = f"bookfile/{f'{fileid}' if fileid is not None else 'bulk'}"
-        return await self._async_request(command, data=data, method=HTTPMethod.DELETE)
-
     async def async_lookup_book(
-        self, term: str, booktype: str = "isbn"
+        self, term: str, booktype: ReadarrBookTypes = ReadarrBookTypes.ISBN
     ) -> list[ReadarrBookLookup]:
         """Search for new books using a term, goodreads ID, isbn or asin."""
-        book_id_types = ["goodreads", "isbn", "asin"]
-        if booktype not in book_id_types:
-            raise ValueError(f"Invalid book id type. Expected one of: {book_id_types}")
         return await self._async_request(
             "book/lookup",
-            params={"term": f"{booktype}:{term}"},
+            params={TERM: f"{booktype}:{term}"},
             datatype=ReadarrBookLookup,
         )
 
-    async def async_add_bookself(self, data: ReadarrBookshelf) -> HTTPResponse:
+    async def async_add_bookshelf(
+        self, data: ReadarrBookshelf
+    ) -> ReadarrBookshelf:  # verify
         """Add a bookshelf to the database."""
         return await self._async_request("bookshelf", data=data, method=HTTPMethod.POST)
 
     async def async_get_calendar(  # pylint: disable=too-many-arguments
         self,
-        start_date: datetime,
-        end_date: datetime,
-        calendarid: int | None = None,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        bookid: int | None = None,
         unmonitored: bool = False,
         includeauthor: bool = False,
     ) -> ReadarrCalendar | list[ReadarrCalendar]:
-        """Get calendar items."""
+        """Get calendar items.
+
+        bookid: Specify to get calendar of releases from book id
+        """
         params = {
-            "start": start_date.strftime("%Y-%m-%d"),
-            "end": end_date.strftime("%Y-%m-%d"),
             "unmonitored": str(unmonitored),
             "includeAuthor": str(includeauthor),
         }
+        if start_date:
+            params["start"] = start_date.strftime("%Y-%m-%d")
+        if end_date:
+            params["end"] = end_date.strftime("%Y-%m-%d")
         return await self._async_request(
-            f"calendar{f'/{calendarid}' if calendarid is not None else ''}",
+            f"calendar{'' if bookid is None else f'/{bookid}'}",
             params=params,
             datatype=ReadarrCalendar,
         )
 
     # /api/v1/calendar/readarr.ics not working
 
-    async def async_get_delay_profiles(
-        self, profileid: int | None = None
-    ) -> list[ReadarrDelayProfile]:
-        """Get all delay profiles."""
-        return await self._async_request(
-            f"delayprofile{f'/{profileid}' if profileid is not None else ''}",
-            datatype=ReadarrDelayProfile,
-        )
-
-    async def async_add_delay_profile(
-        self, data: ReadarrDelayProfile
-    ) -> ReadarrDelayProfile:
-        """Add delay profiles."""
-        return await self._async_request(
-            "delayprofile",
-            data=data,
-            datatype=ReadarrDelayProfile,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_edit_delay_profile(
-        self, data: ReadarrDelayProfile
-    ) -> ReadarrDelayProfile:
-        """Edit delay profiles."""
-        return await self._async_request(
-            f"delayprofile/{data.id}",
-            data=data,
-            datatype=ReadarrDelayProfile,
-            method=HTTPMethod.PUT,
-        )
-
-    async def async_delete_delay_profile(self, profileid: int) -> HTTPResponse:
-        """Delete delay profiles."""
-        return await self._async_request(
-            f"delayprofile/{profileid}", method=HTTPMethod.DELETE
-        )
-
-    async def async_delay_profile_reorder(
-        self, profileid: int, afterid: int | None = None
-    ) -> list[ReadarrDelayProfile]:
-        """Reorder delay profiles."""
-        return await self._async_request(
-            f"delayprofile/reorder/{profileid}",
-            params={"afterId": afterid},
-            datatype=ReadarrDelayProfile,
-            method=HTTPMethod.PUT,
-        )
-
-    @api_command("config/development", datatype=ReadarrDevelopmentConfig)
     async def async_get_development_config(self) -> ReadarrDevelopmentConfig:
         """Get development config."""
+        return await self._async_request(
+            "config/development", datatype=ReadarrDevelopmentConfig
+        )
 
-    # Documented, does not seem to work.
     async def async_edit_development_config(
         self, data: ReadarrDevelopmentConfig
     ) -> ReadarrDevelopmentConfig:
         """Edit development config."""
         return await self._async_request(
-            f"config/development/{data.id}",
+            "config/development",
             data=data,
             datatype=ReadarrDevelopmentConfig,
             method=HTTPMethod.PUT,
         )
 
-    async def async_get_history(
+    async def async_get_history(  # pylint: disable=too-many-arguments
         self,
+        page: int = 1,
+        page_size: int = 20,
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: ReadarrSortKeys = ReadarrSortKeys.DATE,
+        event_type: ReadarrEventType | None = None,
     ) -> ReadarrHistory:
-        """Get history."""
-        return await self._async_request("history", datatype=ReadarrHistory)
+        """Get history.
 
-    # history/since / history/author
+        Args:
+            page: Page to be returned.
+            page_size: Number of results per page.
+            sort_key: date, id, title, sourcetitle, path, ratings, or quality
+                    (Others do not apply)
+        """
+        params = {
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
+        }
+        if event_type and event_type in ReadarrEventType:
+            params[EVENT_TYPE] = event_type.value
+        return await self._async_request(
+            "history", params=params, datatype=ReadarrHistory
+        )
 
-    # history/failed POST does not seem to work
+    async def async_get_history_since(
+        self,
+        event_type: ReadarrEventType | None = None,
+        date: datetime | None = None,
+        authorid: int | None = None,
+    ) -> list[ReadarrBookHistory]:
+        """Get history since specified date.
+
+        Args:
+            authorid: include to search history by author id (date will not apply)
+        Radarr permits a naked query but its required here to avoid excessively large
+        data sets where filtering should be used instead
+        """
+        if date is None and authorid is None:
+            raise ArrException(self, "Either date or authorid is required")
+        params: dict[str, int | str] = {}
+        if event_type and event_type in ReadarrEventType:
+            params[EVENT_TYPE] = event_type.value
+        if isinstance(date, datetime):
+            params[DATE] = date.strftime("%Y-%m-%d")
+        elif authorid is not None:
+            params[AUTHOR_ID] = authorid
+        return await self._async_request(
+            f"history/{'since' if isinstance(date, datetime) else 'author'}",
+            params=params,
+            datatype=ReadarrBookHistory,
+        )
 
     async def async_get_import_lists(
         self, listid: int | None = None
-    ) -> ReadarrImportlist | list[ReadarrImportlist]:
+    ) -> ReadarrImportList | list[ReadarrImportList]:
         """Get import lists."""
         return await self._async_request(
-            f"importlist{f'/{listid}' if listid is not None else ''}",
-            datatype=ReadarrImportlist,
+            f"importlist{'' if listid is None else f'/{listid}'}",
+            datatype=ReadarrImportList,
         )
 
     async def async_edit_import_list(
-        self, data: ReadarrImportlist
-    ) -> ReadarrImportlist:
+        self, data: ReadarrImportList
+    ) -> ReadarrImportList:
         """Edit import list."""
         return await self._async_request(
-            f"importlist{data.id}",
+            "importlist",
             data=data,
-            datatype=ReadarrImportlist,
+            datatype=ReadarrImportList,
             method=HTTPMethod.PUT,
         )
 
-    async def async_add_import_list(self, data: ReadarrImportlist) -> ReadarrImportlist:
+    async def async_add_import_list(self, data: ReadarrImportList) -> ReadarrImportList:
         """Add import list."""
         return await self._async_request(
-            "importlist", data=data, datatype=ReadarrImportlist, method=HTTPMethod.POST
+            "importlist", data=data, datatype=ReadarrImportList, method=HTTPMethod.POST
+        )
+
+    async def async_test_import_lists(
+        self, data: ReadarrImportList | None = None
+    ) -> bool:
+        """Test all import lists."""
+        _res = await self._async_request(
+            f"importlist/test{ALL if data is None else ''}",
+            data=None if data is None else data,
+            method=HTTPMethod.POST,
+        )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
+
+    async def async_importlist_action(
+        self, data: Indexer, action: str = "newznabCategories"
+    ) -> ReadarrImportListOptions:
+        """Perform import list action. Other actions may be possible."""
+        return await self._async_request(
+            f"importlist/action/{action}",
+            data=data,
+            datatype=ReadarrImportListOptions,
+            method=HTTPMethod.POST,
         )
 
     async def async_get_metadata_profiles(
@@ -591,56 +552,54 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> ReadarrMetadataProfile | list[ReadarrMetadataProfile]:
         """Get metadata profiles."""
         return await self._async_request(
-            f"metadataprofile{f'/{profileid}' if profileid is not None else ''}",
+            f"metadataprofile{'' if profileid is None else f'/{profileid}'}",
             datatype=ReadarrMetadataProfile,
         )
 
-    async def async_delete_metadata_profile(
-        self, profileid: int | None = None
-    ) -> HTTPResponse:
-        """Delete a metadata profile."""
-        return await self._async_request(
-            f"metadataprofile/{profileid}",
-            method=HTTPMethod.DELETE,
-        )
-
     async def async_edit_metadata_profile(
         self, data: ReadarrMetadataProfile
     ) -> ReadarrMetadataProfile:
         """Edit a metadata profile."""
         return await self._async_request(
-            f"metadataprofile/{data.id}",
+            "metadataprofile",
             data=data,
             datatype=ReadarrMetadataProfile,
             method=HTTPMethod.PUT,
         )
 
-    # metadataprofile/schema, not that useful
-
-    async def async_get_metadata_provider_configs(
-        self, providerid: int | None = None
-    ) -> ReadarrMetadataProviderConfig | list[ReadarrMetadataProviderConfig]:
-        """Get metadata provider configs."""
+    async def async_add_metadata_profile(
+        self, data: ReadarrMetadataProfile
+    ) -> ReadarrMetadataProfile:
+        """Add a metadata profile."""
         return await self._async_request(
-            f"metadataprovider{f'/{providerid}' if providerid is not None else ''}",
-            datatype=ReadarrMetadataProviderConfig,
+            "metadataprofile",
+            data=data,
+            datatype=ReadarrMetadataProfile,
+            method=HTTPMethod.POST,
         )
 
-    async def async_edit_metadata_provider_config(
-        self,
-        data: ReadarrMetadataProviderConfig,
-    ) -> ReadarrMetadataProviderConfig:
-        """Edit a metadata provider config."""
+    async def async_get_metadata_provider(self) -> ReadarrMetadataProvider:
+        """Get metadata provider."""
         return await self._async_request(
-            f"metadataprovider/{data.id}",
+            "config/metadataprovider",
+            datatype=ReadarrMetadataProvider,
+        )
+
+    async def async_edit_metadata_provider(
+        self,
+        data: ReadarrMetadataProvider,
+    ) -> ReadarrMetadataProvider:
+        """Edit metadata provider."""
+        return await self._async_request(
+            "config/metadataprovider",
             data=data,
-            datatype=ReadarrMetadataProviderConfig,
+            datatype=ReadarrMetadataProvider,
             method=HTTPMethod.PUT,
         )
 
-    @api_command("config/naming", datatype=ReadarrNamingConfig)
     async def async_get_naming_config(self) -> ReadarrNamingConfig:
         """Get information about naming configuration."""
+        return await self._async_request("config/naming", datatype=ReadarrNamingConfig)
 
     async def async_edit_naming_config(
         self, data: ReadarrNamingConfig
@@ -661,7 +620,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get notification matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"notification{f'/{notifyid}' if notifyid is not None else ''}",
+            f"notification{'' if notifyid is None else f'/{notifyid}'}",
             datatype=ReadarrNotification,
         )
 
@@ -670,7 +629,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> ReadarrNotification:
         """Edit a notification."""
         return await self._async_request(
-            f"notification/{data.id}",
+            NOTIFICATION,
             data=data,
             datatype=ReadarrNotification,
             method=HTTPMethod.PUT,
@@ -681,23 +640,30 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> ReadarrNotification:
         """Add a notification."""
         return await self._async_request(
-            "notification",
+            NOTIFICATION,
             data=data,
             datatype=ReadarrNotification,
             method=HTTPMethod.POST,
         )
 
-    async def async_test_notification(self, data: ReadarrNotification) -> HTTPResponse:
+    async def async_test_notifications(
+        self, data: ReadarrNotification | None = None
+    ) -> bool:
         """Test a notification configuration."""
-        return await self._async_request(
-            "notification/test",
-            data=data,
+        _res = await self._async_request(
+            f"notification/test{ALL if data is None else ''}",
+            data=None if data is None else data,
             method=HTTPMethod.POST,
         )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
 
     async def async_parse(self, title: str) -> ReadarrParse:
         """Return the movie with matching file name."""
-        params = {"title": title}
+        params = {TITLE: title}
         return await self._async_request("parse", params=params, datatype=ReadarrParse)
 
     async def async_get_release(
@@ -706,9 +672,9 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Search indexers for specified fields."""
         params = {}
         if authorid is not None:
-            params["authorId"] = authorid
+            params[AUTHOR_ID] = authorid
         if bookid is not None:
-            params["bookId"] = bookid
+            params[BOOK_ID] = bookid
         return await self._async_request(
             "release", params=params, datatype=ReadarrRelease
         )
@@ -746,17 +712,48 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Get files matching specified id that are not properly renamed yet."""
         return await self._async_request(
             "rename",
-            params={"authorId": authorid, "bookId": bookid},
+            params={AUTHOR_ID: authorid, BOOK_ID: bookid},
             datatype=ReadarrRename,
         )
 
+    async def async_get_manual_import(  # pylint: disable=too-many-arguments
+        self,
+        downloadid: str,
+        authorid: int = 0,
+        folder: str | None = None,
+        filterexistingfiles: bool = True,
+        replaceexistingfiles: bool = True,
+    ) -> list[ReadarrManualImport]:
+        """Get manual import."""
+        params = {
+            AUTHOR_ID: authorid,
+            "downloadId": downloadid,
+            "filterExistingFiles": str(filterexistingfiles),
+            "folder": folder if folder is not None else "",
+            "replaceExistingFiles": str(replaceexistingfiles),
+        }
+        return await self._async_request(
+            "manualimport", params=params, datatype=ReadarrManualImport
+        )
+
+    async def async_edit_manual_import(
+        self, data: ReadarrManualImport
+    ) -> list[ReadarrManualImport]:
+        """Get manual import."""
+        return await self._async_request(
+            "manualimport",
+            data=data,
+            datatype=ReadarrManualImport,
+            method=HTTPMethod.PUT,
+        )
+
     async def async_get_retag(
         self, authorid: int = 0, bookid: int = 0
     ) -> list[ReadarrRetag]:
         """Get retag."""
         return await self._async_request(
             "retag",
-            params={"authorId": authorid, "bookId": bookid},
+            params={AUTHOR_ID: authorid, BOOK_ID: bookid},
             datatype=ReadarrRetag,
         )
 
@@ -764,7 +761,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Search for books."""
         return await self._async_request(
             "search",
-            params={"term": term},
+            params={TERM: term},
             datatype=ReadarrSearch,
         )
 
@@ -772,7 +769,7 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """Get series."""
         return await self._async_request(
             "series",
-            params={"authorId": authorid},
+            params={AUTHOR_ID: authorid},
             datatype=ReadarrSeries,
         )
 
@@ -784,6 +781,15 @@ class ReadarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get tag details matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"tag/detail{f'/{tagid}' if tagid is not None else ''}",
+            f"tag/detail{'' if tagid is None else f'/{tagid}'}",
             datatype=ReadarrTagDetails,
         )
+
+    async def async_edit_root_folder(self, data: RootFolder) -> RootFolder:
+        """Edit information about root folders."""
+        return await self._async_request(
+            "rootfolder",
+            data=data,
+            datatype=RootFolder,
+            method=HTTPMethod.PUT,
+        )
