@@ -2,23 +2,43 @@
 from __future__ import annotations
 
 from datetime import datetime
-from typing import TYPE_CHECKING, Any
-from urllib.parse import quote
+from typing import Any
 
-from .const import HTTPMethod
-from .decorator import api_command
-from .exceptions import ArrInvalidCommand, ArrResourceNotFound
-from .models.request import Command
-from .request_client import RequestClient
+from aiohttp.client import ClientSession
 
-from .models.sonarr import (  # isort:skip
+from .const import (
+    ALL,
+    DATE,
+    EPISODE_ID,
+    EVENT_TYPE,
+    IS_VALID,
+    NOTIFICATION,
+    PAGE,
+    PAGE_SIZE,
+    PATH,
+    SERIES_ID,
+    SORT_DIRECTION,
+    SORT_KEY,
+    TERM,
+    TITLE,
+    HTTPMethod,
+)
+from .exceptions import ArrException
+from .models.host_configuration import PyArrHostConfiguration
+from .models.request import Command, SortDirection
+from .models.sonarr import (
     SonarrBlocklist,
     SonarrCalendar,
+    SonarrCommands,
     SonarrEpisode,
     SonarrEpisodeFile,
-    SonarrEpisodeFileQuailty,
+    SonarrEpisodeHistory,
+    SonarrEpisodeMonitor,
+    SonarrEventType,
     SonarrHistory,
     SonarrImportList,
+    SonarrLanguage,
+    SonarrManualImport,
     SonarrNamingConfig,
     SonarrNotification,
     SonarrParse,
@@ -26,23 +46,22 @@ from .models.sonarr import (  # isort:skip
     SonarrQueueDetail,
     SonarrRelease,
     SonarrRename,
+    SonarrSeasonPass,
     SonarrSeries,
+    SonarrSeriesAdd,
     SonarrSeriesLookup,
-    SonarrSeriesUpdateParams,
+    SonarrSortKeys,
     SonarrTagDetails,
     SonarrWantedMissing,
 )
-
-if TYPE_CHECKING:
-    from aiohttp.client import ClientSession
-
-    from .const import HTTPResponse
-    from .models.host_configuration import PyArrHostConfiguration
+from .request_client import RequestClient
 
 
 class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     """API client for Sonarr endpoints."""
 
+    __name__ = "Sonarr"
+
     def __init__(  # pylint: disable=too-many-arguments
         self,
         host_configuration: PyArrHostConfiguration | None = None,
@@ -57,15 +76,14 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         base_api_path: str | None = None,
         request_timeout: float = 30,
         raw_response: bool = False,
-        redact: bool = True,
-        api_ver: str | None = None,
+        api_ver: str = "v3",
     ) -> None:
         """Initialize Sonarr API."""
         super().__init__(
             port,
             request_timeout,
             raw_response,
-            redact,
+            api_ver,
             host_configuration,
             session,
             hostname,
@@ -75,19 +93,19 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             ssl,
             verify_ssl,
             base_api_path,
-            api_ver,
         )
 
     async def async_get_episode_files(
-        self, seriesid: int, episodeid: int | None = None
+        self, entryid: int, series: bool = False
     ) -> SonarrEpisodeFile | list[SonarrEpisodeFile]:
-        """Get information about episode files from series.
+        """Get information about an episode file.
 
-        episodeId: Get episode files matching series. Leave blank for all episodes.
+        episodeId: Get episode files matching id.
+        series: Make True to search by series.
         """
         return await self._async_request(
-            f"episodefile{f'/{episodeid}' if episodeid is not None else ''}",
-            params={"seriesId": seriesid},
+            f"episodefile{'' if series else f'/{entryid}'}",
+            params={SERIES_ID: entryid} if series else None,
             datatype=SonarrEpisodeFile,
         )
 
@@ -95,21 +113,24 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         self,
         page: int = 1,
         page_size: int = 20,
-        sort_direction: str = "ascending",
-        sort_key: str = "timeLeft",
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: SonarrSortKeys = SonarrSortKeys.TIMELEFT,
         include_unknown_series_items: bool = False,
         include_series: bool = False,
         include_episode: bool = False,
     ) -> SonarrQueue:
-        """Get information about download queue."""
+        """Get information about download queue.
+
+        page: Page number to return.
+        page_size: Number of items per page.
+        id: Filter to a specific episode ID.
+        """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
-            "includeUnknownSeriesItems": str(
-                include_unknown_series_items
-            ),  # Unverified
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
+            "includeUnknownSeriesItems": str(include_unknown_series_items),
             "includeSeries": str(include_series),
             "includeEpisode": str(include_episode),
         }
@@ -123,9 +144,7 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> list[SonarrQueueDetail]:
         """Get details of all items in queue."""
         params = {
-            "includeUnknownSeriesItems": str(
-                include_unknown_series_items
-            ),  # Unverified
+            "includeUnknownSeriesItems": str(include_unknown_series_items),
             "includeSeries": str(include_series),
             "includeEpisode": str(include_episode),
         }
@@ -135,96 +154,85 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             datatype=SonarrQueueDetail,
         )
 
-    async def async_get_calendar(
-        self, start_date: datetime | None = None, end_date: datetime | None = None
-    ) -> list[SonarrCalendar]:
-        """Get upcoming episodes, if start/end are not supplied episodes airing today.
+    async def async_get_calendar(  # pylint: disable=too-many-arguments
+        self,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        episodeid: int | None = None,
+        include_series: bool = False,
+        include_episode_file: bool = False,
+        include_unmonitored: bool = False,
+    ) -> SonarrCalendar | list[SonarrCalendar]:
+        """Get upcoming episodes.
 
-        and tomorrow will be returned
+        If start/end not supplied episodes airing today and tomorrow will be returned
+        episodeid: Specify to get calendar info from episode id
         """
         params = {}
+
         if start_date:
-            params["start"] = (start_date.strftime("%Y-%m-%d"),)
+            params["start"] = start_date.strftime("%Y-%m-%d")
         if end_date:
-            params["end"] = (end_date.strftime("%Y-%m-%d"),)
+            params["end"] = end_date.strftime("%Y-%m-%d")
+
+        if include_series:
+            params["includeSeries"] = "True"
+
+        if include_episode_file:
+            params["includeEpisodeFile"] = "True"
+
+        if include_unmonitored:
+            params["includeUnmonitored"] = "True"
 
         return await self._async_request(
-            "calendar", params=params, datatype=SonarrCalendar
+            f"calendar{'' if episodeid is None else f'/{episodeid}'}",
+            params=params,
+            datatype=SonarrCalendar,
         )
 
-    async def async_command_refresh_series(
-        self, seriesid: int | None = None
-    ) -> Command:
-        """Send refresh series command."""
-        data = {"name": "RefreshSeries"}
-        if seriesid is not None:
-            data["seriesId"] = str(seriesid)
-        return await self._async_request(
-            "command",
-            data=data,
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_rescan_series(self, seriesid: int | None = None) -> Command:
-        """Send rescan series command."""
-        data = {"name": "RefreshSeries"}
-        if seriesid is not None:
-            data["seriesId"] = str(seriesid)
-        return await self._async_request(
-            "command",
-            data=data,
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_episode_search(self, episodeids: int) -> Command:
-        """Send episode search command."""
-        return await self._async_request(
-            "command",
-            data={"name": "EpisodeSearch", "episodeIds": episodeids},
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_season_search(
-        self, seriesid: int, seasonnumber: int
-    ) -> Command:
-        """Send season search command."""
-        return await self._async_request(
-            "command",
-            data={
-                "name": "SeasonSearch",
-                "seriesId": seriesid,
-                "seasonNumber": seasonnumber,
-            },
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_series_search(self, seriesid: int) -> Command:
-        """Send series search command."""
-        return await self._async_request(
-            "command",
-            data={"name": "SeriesSearch", "seriesId": seriesid},
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_downloaded_episode_scan(
+    async def async_sonarr_command(  # pylint: disable=too-many-arguments
         self,
-        path: str,
-        clientid: str | None = None,
-        importmode: str = "Copy",
+        command: SonarrCommands,
+        clientid: int | None = None,
+        copymode: bool = True,
+        episodeids: list[int] | None = None,
+        files: list[int] | None = None,
+        path: str | None = None,
+        season: int | None = None,
+        seriesid: int | list[int] | None = None,
     ) -> Command:
-        """Send series search command.
+        """Send a command to Sonarr.
 
-        importMode: Copy or Hardlink depending on Sonarr configuration
+        Specify clientid for DownloadedEpisodesScan (Optional)
+        Specify episodeIds for EpisodeSearch
+        Specify files for RenameFiles
+        Specify path for DownloadedEpisodesScan (Optional)
+        Specify season for SeasonSearch
+        Specify seriesId for:
+            RefreshSeries (Optional),
+            RenameSeries (list[int]),
+            RescanSeries (Optional),
+            SeasonSearch,
+            SeriesSearch
         """
-        data = {"name": "DownloadedEpisodesScan", "path": path}
-        data["importMode"] = importmode
+        data: dict[str, str | int | list[int]] = {"name": command.value}
         if clientid is not None:
             data["downloadClientId"] = clientid
+        if episodeids is not None:
+            data["episodeIds"] = episodeids
+        if files is not None:
+            data["files"] = files
+        if path is not None:
+            data[PATH] = path
+        if seriesid is not None:
+            if command == SonarrCommands.RENAME_SERIES:
+                data["seriesIds"] = seriesid
+            else:
+                data[SERIES_ID] = seriesid
+        if command is SonarrCommands.DOWNLOADED_EPISODES_SCAN:
+            data["importMode"] = "Copy" if copymode else "Move"
+        if season is not None:
+            data["seasonNumber"] = season
         return await self._async_request(
             "command",
             data=data,
@@ -232,68 +240,41 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             method=HTTPMethod.POST,
         )
 
-    async def async_command_rename_files(self, files: list[int]) -> Command:
-        """Send rename files command."""
+    async def async_get_episodes(
+        self, entryid: int, series: bool = False
+    ) -> SonarrEpisode | list[SonarrEpisode]:
+        """Get all episodes from a given series or episode id."""
         return await self._async_request(
-            "command",
-            data={"name": "RenameFiles", "files": files},
-            datatype=Command,
-            method=HTTPMethod.POST,
+            f"episode{'' if series else f'/{entryid}'}",
+            params={SERIES_ID: entryid} if series else None,
+            datatype=SonarrEpisode,
         )
 
-    async def async_command_rename_series(self, seriesids: list[int]) -> Command:
-        """Send rename series command."""
-        return await self._async_request(
-            "command",
-            data={"name": "RenameSeries", "seriesIds": seriesids},
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_backup(self) -> Command:
-        """Send backup command."""
-        return await self._async_request(
-            "command",
-            data={"name": "Backup"},
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_command_missing_search(self) -> Command:
-        """Send missing episode search command."""
-        return await self._async_request(
-            "command",
-            data={"name": "missingEpisodeSearch"},
-            datatype=Command,
-            method=HTTPMethod.POST,
-        )
-
-    async def async_get_episodes_by_series_id(self, seriesid: int) -> SonarrEpisode:
-        """Get all episodes from a given series ID."""
-        return await self._async_request("episode", params={"seriesId": seriesid})
-
-    async def async_get_episode_by_id(self, episodeid: int) -> SonarrEpisode:
-        """Get a specific episode by database id."""
-        return await self._async_request(f"episode/{episodeid}", datatype=SonarrEpisode)
-
     async def async_edit_episode(self, data: SonarrEpisode) -> SonarrEpisode:
         """Edit given episodes, currently only monitored is changed."""
         return await self._async_request(
-            "episode", data=data, datatype=SonarrEpisode, method=HTTPMethod.PUT
+            f"episode/{data.id}",
+            data=data,
+            datatype=SonarrEpisode,
+            method=HTTPMethod.PUT,
         )
 
     async def async_edit_episode_file_quality(
-        self, fileid: int, data: SonarrEpisodeFileQuailty
+        self, data: SonarrEpisodeFile
     ) -> SonarrEpisodeFile:
-        """Edit the quality of the episode file and returns the episode file."""
+        """Edit an episode file quality.
+
+        Quality only works, currently.
+        Quality.id and Revision.version
+        """
         return await self._async_request(
-            f"episodefile/{fileid}",
+            "episodefile",
             data=data,
             datatype=SonarrEpisodeFile,
             method=HTTPMethod.PUT,
         )
 
-    async def async_delete_episode_file(self, fileid: int) -> dict:
+    async def async_delete_episode_file(self, fileid: int) -> None:
         """Delete the episode file with corresponding id."""
         return await self._async_request(
             f"episodefile/{fileid}", method=HTTPMethod.DELETE
@@ -301,54 +282,87 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
 
     async def async_get_history(  # pylint: disable=too-many-arguments
         self,
-        sort_date: bool = True,
         page: int = 1,
         page_size: int = 10,
-        sort_asc: bool = False,
+        sort_key: SonarrSortKeys = SonarrSortKeys.DATE,
         recordid: int | None = None,
+        event_type: SonarrEventType | None = None,
     ) -> SonarrHistory:
         """Get history (grabs/failures/completed).
 
         Args:
-            sort_date: date or series.title.
+            sort_key: series.title, id, seriesid, episodeid, date,
+                    quality, or sourcetitle (others do not apply)
             page: Page number to return.
             page_size: Number of items per page.
-            sort_asc: Sort items in ascending order.
             id: Filter to a specific episode ID.
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDir": "asc" if sort_asc is True else "desc",
-            "sortKey": "date" if sort_date is True else "series.title",
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_KEY: sort_key.value,
         }
-        if recordid:
-            params["episodeId"] = recordid
+        if event_type and event_type in SonarrEventType:
+            params[EVENT_TYPE] = event_type.value
+        if recordid is not None:
+            params[EPISODE_ID] = recordid
         return await self._async_request(
             "history", datatype=SonarrHistory, params=params
         )
 
-    async def async_get_wanted(
+    async def async_get_history_since(
+        self,
+        date: datetime | None = None,
+        seriesid: int | None = None,
+        event_type: SonarrEventType | None = None,
+    ) -> list[SonarrEpisodeHistory]:
+        """Get history since specified date.
+
+        seriesid: include to search history by series id (date will not apply)
+        """
+        if date is None and seriesid is None:
+            raise ArrException(self, "Either date or seriesid is required")
+        params: dict[str, int | str] = {}
+        if isinstance(date, datetime):
+            params[DATE] = date.strftime("%Y-%m-%d")
+        elif seriesid is not None:
+            params[SERIES_ID] = seriesid
+        if event_type and event_type in SonarrEventType:
+            params[EVENT_TYPE] = event_type.value
+        return await self._async_request(
+            f"history/{'since' if isinstance(date, datetime) else 'series'}",
+            params=params,
+            datatype=SonarrEpisodeHistory,
+        )
+
+    # POST history/failed
+
+    async def async_get_wanted(  # pylint: disable=too-many-arguments
         self,
-        sort_date: bool = True,
         page: int = 1,
         page_size: int = 10,
-        sort_asc: bool = True,
+        sort_key: SonarrSortKeys = SonarrSortKeys.AIR_DATE_UTC,
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        include_series: bool = False,
     ) -> SonarrWantedMissing:
         """Get missing episode (episodes without files).
 
         Args:
-            sort_date: airDateUtc or series.title.
+            sort_key: series.title, id, seriesid, airdateutc, ratings, or title
+            (others do not apply)
             page: Page number to return.
             page_size: Number of items per page.
-            sort_asc: Sort items in ascending order.
         """
         params = {
-            "sortKey": "airDateUtc" if sort_date is True else "series.title",
-            "page": page,
-            "pageSize": page_size,
-            "sortDir": "asc" if sort_asc is True else "desc",
+            SORT_KEY: sort_key.value,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
         }
+
+        if include_series:
+            params["includeSeries"] = "True"
+
         return await self._async_request(
             "wanted/missing",
             params=params,
@@ -369,47 +383,54 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         path: file path of series / episode
         """
         if title is None and path is None:
-            raise ArrInvalidCommand(self, "A title or path must be specified")
+            raise ArrException(message="A title or path must be specified")
         params = {}
         if title is not None:
-            params["title"] = title
+            params[TITLE] = title
         if path is not None:
-            params["path"] = path
+            params[PATH] = path
         return await self._async_request("parse", params=params, datatype=SonarrParse)
 
-    async def async_get_release(self, episodeid: int) -> list[SonarrRelease]:
-        """Get a release with a specific episode ID."""
+    async def async_get_release(
+        self, episodeid: int | None = None
+    ) -> list[SonarrRelease]:
+        """Query indexers for latest releases.
+
+        episodeid: included in API docs but does not seem to do anything
+        """
         return await self._async_request(
             "release",
-            params={"episodeId": episodeid},
+            params=None if episodeid is None else {EPISODE_ID: episodeid},
             datatype=SonarrRelease,
         )
 
-    async def async_download_release(
-        self, guid: str, indexerid: int
-    ) -> list[SonarrRelease]:
+    async def async_download_release(self, data: SonarrRelease) -> SonarrRelease:
         """Add a previously searched release to the download client.
 
         If the release is
         still in the search cache (30 minute cache). If the release is not found
         in the cache it will return a 404.
 
-        guid: Recently searched result guid
+        Only the guid and indexerId are required
+        but a full SonarrRelease object will work
         """
         return await self._async_request(
             "release",
-            data={"guid": guid, "indexerId": indexerid},
+            data=data,
             datatype=SonarrRelease,
             method=HTTPMethod.POST,
         )
 
-    # Only works if an id is associated with the release
-    async def async_get_pushed_release(self, releaseid: str) -> SonarrRelease:
-        """Get release previously pushed by below method."""
+    async def async_push_release(self, data: SonarrRelease) -> list[SonarrRelease]:
+        """Add a release.
+
+        If the title is wanted, Sonarr will grab it.
+        """
         return await self._async_request(
             "release/push",
-            params={"id": releaseid},
+            data=data,
             datatype=SonarrRelease,
+            method=HTTPMethod.POST,
         )
 
     async def async_get_series(
@@ -420,66 +441,20 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         series ID if one is found.
         """
         return await self._async_request(
-            f"series{f'/{seriesid}' if seriesid is not None else ''}",
+            f"series{'' if seriesid is None else f'/{seriesid}'}",
             datatype=SonarrSeries,
         )
 
-    async def async_get_series_episodes(
-        self, seriesid: int, episodeid: int | None = None
-    ) -> list[SonarrEpisode]:
-        """Get information about episodes from series."""
-        response = await self._async_request(
-            f"episode{f'/{episodeid}' if episodeid is not None else ''}",
-            params={"seriesId": seriesid},
-            datatype=SonarrEpisode,
-        )
-        if "message" in response and response["message"] == "NotFound":
-            raise ArrResourceNotFound
-        if "seriesId" in response and response["seriesId"] != seriesid:
-            raise ArrResourceNotFound
-        return response
-
-    async def async_add_series(  # pylint: disable=too-many-arguments
-        self,
-        tvdb_id: int,
-        quality_profile_id: int,
-        root_dir: str,
-        season_folder: bool = True,
-        monitored: bool = True,
-        ignore_episodes_with_files: bool = False,
-        ignore_episodes_without_files: bool = False,
-        search_for_missing_episodes: bool = False,
-    ) -> SonarrSeries:
-        """Add a new series to your collection.
-
-        Args:
-            tvdb_id: TDDB Id
-            quality_profile_id: Database id for quality profile
-            root_dir: Root folder location, full path will be created from this
-            season_folder: Create a folder for each season.
-            monitored: Monitor this series.
-            ignore_episodes_with_files: Ignore any episodes with existing files.
-            ignore_episodes_without_files: Ignore any episodes without existing files.
-            search_for_missing_episodes: Search for missing episodes to download.
-        """
-        series_json = await self._async_construct_series_json(
-            tvdb_id,
-            quality_profile_id,
-            root_dir,
-            season_folder,
-            monitored,
-            ignore_episodes_with_files,
-            ignore_episodes_without_files,
-            search_for_missing_episodes,
-        )
+    async def async_add_series(self, data: SonarrSeriesAdd) -> SonarrSeries:
+        """Add a new series to your collection."""
         return await self._async_request(
             "series",
-            data=series_json,
+            data=data,
             datatype=SonarrSeries,
             method=HTTPMethod.POST,
         )
 
-    async def async_edit_series(self, data: SonarrSeriesUpdateParams) -> SonarrSeries:
+    async def async_edit_series(self, data: SonarrSeries) -> SonarrSeries:
         """Edit an existing series."""
         return await self._async_request(
             "series",
@@ -497,10 +472,12 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         """
         return await self._async_request(
             f"series/{seriesid}",
-            params={"id": seriesid, "deleteFiles": str(delete_files)},
+            params={"deleteFiles": str(delete_files)},
             method=HTTPMethod.DELETE,
         )
 
+    # POST series/import
+
     async def async_lookup_series(
         self, term: str | None = None, seriesid: int | None = None
     ) -> list[SonarrSeriesLookup]:
@@ -509,11 +486,11 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         caching and augmentation proxy.
         """
         if term is None and seriesid is None:
-            raise ArrResourceNotFound(self, "A term or TVDB id must be included")
+            raise ArrException(self, "A term or TVDB id must be included")
         return await self._async_request(
             "series/lookup",
             datatype=SonarrSeriesLookup,
-            params={"term": quote(term, safe="") if term else f"tvdb:{seriesid}"},
+            params={TERM: term if term else f"tvdb:{seriesid}"},
         )
 
     async def async_get_import_lists(
@@ -524,17 +501,14 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get import list matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"importlist{f'/{listid}' if listid is not None else ''}",
+            f"importlist{'' if listid is None else f'/{listid}'}",
             datatype=SonarrImportList,
         )
 
     async def async_edit_import_list(self, data: SonarrImportList) -> SonarrImportList:
         """Edit import list."""
         return await self._async_request(
-            f"importlist{data.id}",
-            data=data,
-            datatype=SonarrImportList,
-            method=HTTPMethod.PUT,
+            "importlist", data=data, datatype=SonarrImportList, method=HTTPMethod.PUT
         )
 
     async def async_add_import_list(self, data: SonarrImportList) -> SonarrImportList:
@@ -543,26 +517,93 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             "importlist", data=data, datatype=SonarrImportList, method=HTTPMethod.POST
         )
 
+    async def async_test_import_lists(
+        self, data: SonarrImportList | None = None
+    ) -> bool:
+        """Test all import lists."""
+        _res = await self._async_request(
+            f"importlist/test{ALL if data is None else ''}",
+            data=None if data is None else data,
+            method=HTTPMethod.POST,
+        )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
+
+    async def async_season_pass(self, data: SonarrSeasonPass) -> None:
+        """Change monitoring status via season pass."""
+        return await self._async_request(
+            "seasonPass", data=data, method=HTTPMethod.POST
+        )
+
+    async def async_episode_monitor(
+        self, episodeids: list[int], monitored: bool = True
+    ) -> list[SonarrEpisodeMonitor]:
+        """Change monitoring status via season pass."""
+        return await self._async_request(
+            "episode/monitor",
+            data={"episodeIds": episodeids, "monitored": str(monitored)},
+            datatype=SonarrEpisodeMonitor,
+            method=HTTPMethod.PUT,
+        )
+
+    async def async_get_languages(  # type: ignore[override]
+        self, langid: int | None = None
+    ) -> SonarrLanguage | list[SonarrLanguage]:
+        """Get language profiles."""
+        return await self._async_request(
+            f"languageprofile{'' if langid is None else f'/{langid}'}",
+            datatype=SonarrLanguage,
+        )
+
+    async def async_edit_language(self, data: SonarrLanguage) -> SonarrLanguage:
+        """Edit language profiles."""
+        return await self._async_request(
+            "languageprofile",
+            data=data,
+            datatype=SonarrLanguage,
+            method=HTTPMethod.PUT,
+        )
+
+    async def async_delete_language(self, languageid: int) -> None:
+        """Delete language profiles."""
+        return await self._async_request(
+            f"languageprofile/{languageid}",
+            datatype=SonarrLanguage,
+            method=HTTPMethod.DELETE,
+        )
+
+    async def async_add_language(self, data: SonarrLanguage) -> SonarrLanguage:
+        """Add language profiles."""
+        return await self._async_request(
+            "languageprofile",
+            data=data,
+            datatype=SonarrLanguage,
+            method=HTTPMethod.POST,
+        )
+
     async def async_get_blocklist(
         self,
         page: int = 1,
         page_size: int = 10,
-        sort_direction: str = "descending",
-        sort_key: str = "date",
+        sort_dir: SortDirection = SortDirection.DEFAULT,
+        sort_key: SonarrSortKeys = SonarrSortKeys.DATE,
     ) -> SonarrBlocklist:
         """Return blocklisted releases.
 
         Args:
             page: Page to be returned.
             page_size: Number of results per page.
-            sort_direction: Direction to sort items.
-            sort_key: Field to sort by.
+            sort_key: series.title, id, seriesid, date, indexer,
+                    message, quality, or sourcetitle (Others do not apply)
         """
         params = {
-            "page": page,
-            "pageSize": page_size,
-            "sortDirection": sort_direction,
-            "sortKey": sort_key,
+            PAGE: page,
+            PAGE_SIZE: page_size,
+            SORT_DIRECTION: sort_dir.value,
+            SORT_KEY: sort_key.value,
         }
         return await self._async_request(
             "blocklist",
@@ -570,9 +611,9 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
             datatype=SonarrBlocklist,
         )
 
-    @api_command("config/naming", datatype=SonarrNamingConfig)
     async def async_get_naming_config(self) -> SonarrNamingConfig:
         """Get information about naming configuration."""
+        return await self._async_request("config/naming", datatype=SonarrNamingConfig)
 
     async def async_edit_naming_config(
         self, data: SonarrNamingConfig
@@ -593,7 +634,7 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get notification matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"notification{f'/{notifyid}' if notifyid is not None else ''}",
+            f"notification{'' if notifyid is None else f'/{notifyid}'}",
             datatype=SonarrNotification,
         )
 
@@ -602,7 +643,7 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> SonarrNotification:
         """Edit a notification."""
         return await self._async_request(
-            f"notification/{data.id}",
+            NOTIFICATION,
             data=data,
             datatype=SonarrNotification,
             method=HTTPMethod.PUT,
@@ -613,28 +654,62 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
     ) -> SonarrNotification:
         """Add a notification."""
         return await self._async_request(
-            "notification",
+            NOTIFICATION,
             data=data,
             datatype=SonarrNotification,
             method=HTTPMethod.POST,
         )
 
-    async def async_test_notification(self, data: SonarrNotification) -> HTTPResponse:
+    async def async_test_notifications(
+        self, data: SonarrNotification | None = None
+    ) -> bool:
         """Test a notification configuration."""
-        return await self._async_request(
-            "notification/test",
-            data=data,
+        _res = await self._async_request(
+            f"notification/test{ALL if data is None else ''}",
+            data=None if data is None else data,
             method=HTTPMethod.POST,
         )
+        if data is None:
+            for item in _res:
+                if item[IS_VALID] is False:
+                    return False
+        return True
 
     async def async_get_rename(self, seriesid: int) -> list[SonarrRename]:
         """Get files matching specified id that are not properly renamed yet."""
         return await self._async_request(
             "rename",
-            params={"seriesId": seriesid},
+            params={SERIES_ID: seriesid},
             datatype=SonarrRename,
         )
 
+    async def async_get_manual_import(
+        self,
+        downloadid: str,
+        folder: str | None = None,
+        filterexistingfiles: bool = True,
+    ) -> list[SonarrManualImport]:
+        """Get manual import."""
+        params = {
+            "downloadId": downloadid,
+            "filterExistingFiles": str(filterexistingfiles),
+            "folder": folder if folder is not None else "",
+        }
+        return await self._async_request(
+            "manualimport", params=params, datatype=SonarrManualImport
+        )
+
+    async def async_edit_manual_import(
+        self, data: SonarrManualImport
+    ) -> list[SonarrManualImport]:
+        """Get manual import."""
+        return await self._async_request(
+            "manualimport",
+            data=data,
+            datatype=SonarrManualImport,
+            method=HTTPMethod.PUT,
+        )
+
     async def async_get_tags_details(
         self, tagid: int | None = None
     ) -> SonarrTagDetails | list[SonarrTagDetails]:
@@ -643,63 +718,14 @@ class SonarrClient(RequestClient):  # pylint: disable=too-many-public-methods
         id: Get tag details matching id. Leave blank for all.
         """
         return await self._async_request(
-            f"tag/detail{f'/{tagid}' if tagid is not None else ''}",
+            f"tag/detail{'' if tagid is None else f'/{tagid}'}",
             datatype=SonarrTagDetails,
         )
 
-    async def _async_construct_series_json(  # pylint: disable=too-many-arguments
-        self,
-        tvdb_id: int,
-        quality_profile_id: int,
-        root_dir: str,
-        season_folder: bool = True,
-        monitored: bool = True,
-        ignore_episodes_with_files: bool = False,
-        ignore_episodes_without_files: bool = False,
-        search_for_missing_episodes: bool = False,
-    ) -> dict[str, Any] | None:
-        """Search for new shows on trakt and returns Series JSON to add.
-
-        Args:
-            tvdb_id: TVDB id to search
-            quality_profile_id: Database id for Quality profile
-            root_dir: Root directory for media
-            season_folder: Specify if a season folder should be created.
-            monitored: Specify if the series should be monitored.
-            ignore_episodes_with_files: Ignore episodes that already have files.
-            ignore_episodes_without_files: Ignore episodes that dont have any files.
-            search_for_missing_episodes: Search for any missing episodes and download them.
-        """
-        result = await self.async_lookup_series(seriesid=tvdb_id)
-        if not isinstance(result, list):
-            return None
-        series = result[0]
-        assert isinstance(series, dict)
-        if not monitored and series.get("seasons"):
-            for season in series["seasons"]:
-                season["monitored"] = False
-
-        return {
-            "title": series["title"],
-            "seasons": series["seasons"],
-            "rootFolderPath": root_dir,
-            "qualityProfileId": quality_profile_id,
-            "seasonFolder": season_folder,
-            "monitored": monitored,
-            "tvdbId": tvdb_id,
-            "images": series["images"],
-            "titleSlug": series["titleSlug"],
-            "addOptions": {
-                "ignoreEpisodesWithFiles": ignore_episodes_with_files,
-                "ignoreEpisodesWithoutFiles": ignore_episodes_without_files,
-                "searchForMissingEpisodes": search_for_missing_episodes,
-            },
-        }
-
     async def async_get_localization(self) -> Any:
         """Get localization strings."""
         raise NotImplementedError()
 
-    async def async_get_languages(self, langid: int | None = None) -> Any:
-        """Get import list exclusions."""
+    async def async_delete_metadata_profile(self, profileid: int) -> Any:
+        """Delete a metadata profile."""
         raise NotImplementedError()
